#!/usr/bin/env node
"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const yargs_1 = __importDefault(require("yargs"));
const inquirer_1 = __importDefault(require("inquirer"));
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const child_process_1 = __importDefault(require("child_process"));
const chalk_1 = __importDefault(require("chalk"));
const questions_1 = __importDefault(require("./questions"));
const utils_1 = require("./utils");
const yargsOpts = {
    'customize': {
        alias: 'c'
    }
};
(() => __awaiter(void 0, void 0, void 0, function* () {
    const { _: [name], customize } = yield yargs_1.default.options(yargsOpts).argv;
    const defaults = customize ? {} : (0, utils_1.getAnswers)(questions_1.default, name);
    const _a = yield inquirer_1.default.prompt(questions_1.default, Object.assign(Object.assign({}, defaults), { name })), { options: o } = _a, answers = __rest(_a, ["options"]);
    const inverseOptions = Object.fromEntries((questions_1.default.find(q => q.name === 'options') || { choices: [] }).choices.filter(c => o.indexOf(c.value) < 0).map(c => [c.value, false]));
    const options = Object.assign(Object.assign(Object.assign({}, answers), inverseOptions), Object.fromEntries(o.map((v) => [v, true])));
    console.log('lang', yargs_1.default.locale());
    // create folder for project
    const projectFolder = path_1.default.resolve(process.cwd(), `./${options.name}`);
    if (fs_1.default.existsSync(projectFolder)) {
        console.error(`The directory ${options.name} already exists`);
        process.exit(1);
    }
    fs_1.default.mkdirSync(projectFolder, { recursive: true });
    // generate package.json and write it to the project folder
    const json = yield (0, utils_1.generatePackageJson)(options);
    fs_1.default.writeFileSync(path_1.default.resolve(process.cwd(), `./${options.name}/package.json`), JSON.stringify(json, null, 2));
    // generate dotenv file
    const content = yield (0, utils_1.generateEnvironmentVariables)(options);
    fs_1.default.writeFileSync(path_1.default.resolve(process.cwd(), `./${options.name}/.env`), content);
    // run npm install in the project folder
    if (!process.env.NODE_ENV || process.env.NODE_ENV !== 'development') {
        console.log('Installing packages...');
        child_process_1.default.execSync(`cd ./${options.name} && npm install && cd ../`, { stdio: [0, 1, 2] });
    }
    // construct files
    const filesDir = path_1.default.resolve(__dirname, './files');
    const snippetsDir = path_1.default.resolve(__dirname, './snippets');
    const files = fs_1.default.readdirSync(filesDir);
    files.forEach(fileName => {
        var _a;
        const fileContents = fs_1.default.readFileSync(`${filesDir}/${fileName}`, 'utf8');
        const file = JSON.parse(fileContents);
        if (!(0, utils_1.filterRules)(file.rules, options)) {
            return;
        }
        let imports = file.imports || [];
        const filePath = path_1.default.resolve(process.cwd(), `./${options.name}/${file.path}`);
        const fileDirname = path_1.default.dirname(filePath);
        const content = file.content && !file.snippets ? JSON.stringify(file.content, null, 2) : (_a = file.snippets) === null || _a === void 0 ? void 0 : _a.filter(snippet => (0, utils_1.filterRules)(snippet.rules, options)).map(snippet => {
            const snippetContent = snippet.file ? fs_1.default.readFileSync(`${snippetsDir}/${snippet.file}`, 'utf8') : '';
            imports = imports.concat(snippetContent.match(/^import .*$/gm) || [], snippet.imports || []);
            return snippetContent
                .replace(new RegExp(snippet.replace ? Object.keys(snippet.replace).map(key => key).join('|') : '$^', 'gi'), (match) => match ? options[snippet.replace[match]] : '')
                .replace(/^(declare const|import) .*$/gm, '');
        }).join('\r\n').replace(/[\r\n]\s{2,}[\r\n]/g, '\r\n\r\n');
        if (content || imports.length > 0) {
            fs_1.default.mkdirSync(fileDirname, { recursive: true });
            fs_1.default.writeFileSync(filePath, `${imports.filter((v, i, a) => a.indexOf(v) === i).join('\r\n')}${content}`);
        }
    });
    // git init
    child_process_1.default.execSync(`git init ./${options.name}`, { stdio: [0, 1, 2] });
    console.log(`${chalk_1.default.green("Success!")} Created ${options.name} at ${projectFolder}.\n
In that directory, you can run several commands:
  * npm start: Starts the development server
  * npm run build: Bundles the app into static files for production.\n
We suggest that you begin by typing:
  cd ./${options.name}
  npm start`);
}))();
//# sourceMappingURL=index.js.map